package com.nokia.midp.example.jsr211.texthandler;

import javax.microedition.content.ContentHandlerServer;
import javax.microedition.content.Invocation;
import javax.microedition.content.Registry;
import javax.microedition.content.RequestListener;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

public class TextHandler extends MIDlet implements CommandListener,
		RequestListener {

	/**Confirmation screen title*/
	private static final String CONFIRM = "Confirmation";

	/**Choice list title*/
	private static final String FAVORITE_LINKS = "Favorite Links";

	/** content type constant*/
	private static final String CONTENT_TYPE = "text/plain";

	/** Current class name*/
	private static final String CLASS_NAME = "com.nokia.midp.example.jsr211.texthandler.TextHandler";

	/** Display instance*/
	Display display = null;

	/** Proceed the invocation confirmation*/
	private Command proceedCommand = new Command("Proceed", Command.OK, 1);

	/** Go back to the choice list window*/
	private Command backCommand = new Command("Back", Command.OK, 2);

	/** Exit the application and go back to the invoker*/
	private Command exitCommand = new Command("Exit", Command.BACK, 2);

	/** Current invocation, null if no Invocation. */
	private Invocation invocation;

	/** ContentHandlerServer from which to get requests. */
	private ContentHandlerServer handler;

	/** Access to Registry functions and responses. */
	private Registry registry;

	/**instance to display choice list*/
	ChoiceList choices;

	/**Invocation URL*/
	String url;

	/**
	 * Constructor creates Display instance and create choice list.
	 */
	public TextHandler() {
		display = Display.getDisplay(this);

		//Display the choice list
		choices = new ChoiceList(FAVORITE_LINKS, this);
		display.setCurrent(choices);
	}

	protected void startApp() throws MIDletStateChangeException {
	}

	protected void destroyApp(boolean arg0) {
	}

	protected void pauseApp() {
	}

	/**
	 * Method displays Text Invocation. Current invocation handles
	 * Text Content type "text/plain"
	 * @param invoc invocation with content type "text/plain"
	 */
	void showCurrentInvocation(Invocation invoc) {
		Form textViewerFrom = new TextViewer(invoc, this);
		display.setCurrent(textViewerFrom);
	}

	/**
	 * It does handler registration. Also needs user input to process further.
	 * @param _url invocation url
	 */
	void registerAndConfirm(String _url) {
		url = _url;
		try {
			registry = Registry.getRegistry(TextHandler.CLASS_NAME);
			handler = registry.register(TextHandler.CLASS_NAME,
					new String[] { TextHandler.CONTENT_TYPE }, null, null,
					null, null, null);

			if (handler != null) {
				handler.setListener(this);
			}

		} catch (Exception che) {
			che.printStackTrace();
		}
		if (registry != null && handler != null) {
			confirmationScree("Registration Successful!\n\nWant to Proceed?");
		}
	}

	/**
	 * It displays alert messages
	 * @param message Message to display
	 **/
	private void displayMessage(String message) {
		Alert alert = new Alert("Message", message, null, AlertType.INFO);
		display.setCurrent(alert);
	}

	/**
	 * Confirmation screen to process further.
	 * @param msg Message to display on confirmation screen
	 * */
	private void confirmationScree(String msg) {
		Form confirmForm = new Form(CONFIRM);
		confirmForm.addCommand(backCommand);
		confirmForm.addCommand(proceedCommand);
		confirmForm.append(msg);
		confirmForm.setCommandListener(this);
		display.setCurrent(confirmForm);
	}

	/**
	 * Handle Invocation commands
	 */
	public void commandAction(Command command, Displayable disp) {

		if (command == proceedCommand) {
			doInvoke(url);
		} else if (command == backCommand) {
			display.setCurrent(choices);
		} else if (command == exitCommand) {
			destroyApp(true);
			notifyDestroyed();
		}
	}

	/**
	 * Handles incoming invocation request using the ContentHandlerServer
	 * instance. If there is a current Invocation finish it so the next
	 * Contact can be displayed.
	 */

	public void invocationRequestNotify(ContentHandlerServer server) {

		if (invocation != null) {
			handler.finish(invocation, Invocation.OK);
		}
		invocation = handler.getRequest(false);
		if (invocation != null) {
			showCurrentInvocation(invocation);
		}
	}

	/**
	 * Finish the current invocation
	 * @param invocStatus
	 */
	void doFinish(int invocStatus) {

		if (invocStatus == Invocation.OK) {
			if (invocation != null)
				invocation = null;

			destroyApp(true);
			notifyDestroyed();
		}
	}

	/**
	 * Invoke the URL provided.
	 * @param url http URL
	 */
	void doInvoke(String url) {
		try {
			Invocation invoc = new Invocation(url);
			if (registry.invoke(invoc)) {
				destroyApp(true);
				notifyDestroyed();
			} else {
			}
		} catch (Exception ex) {
			displayMessage("Could not link to " + url);
		}
	}
}
